@@ -47,7 +47,7 @@
 #include "sj-util.h"
 #include "sj-prefs.h"
 #include "sj-play.h"
-#include "sj-volume.h"
+#include "bacon-volume.h"
 
 gboolean on_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data);
 
@@ -69,7 +69,7 @@ GtkWidget *main_window;
 static GtkWidget *title_entry, *artist_entry, *duration_label, *genre_entry;
 static GtkWidget *track_listview, *extract_button, *play_button;
 static GtkWidget *status_bar;
-static GtkWidget *extract_menuitem, *play_menuitem, *select_all_menuitem, *deselect_all_menuitem;
+static GtkWidget *extract_menuitem, *play_menuitem, *next_menuitem, *prev_menuitem, *select_all_menuitem, *deselect_all_menuitem;
 GtkListStore *track_store;
 static BaconMessageConnection *connection;
 GtkCellRenderer *toggle_renderer, *title_renderer, *artist_renderer;
@@ -352,6 +352,8 @@ static void update_ui_for_album (AlbumDetails *album)
     gtk_widget_set_sensitive (extract_menuitem, FALSE);
     gtk_widget_set_sensitive (select_all_menuitem, FALSE);
     gtk_widget_set_sensitive (deselect_all_menuitem, TRUE);
+    gtk_widget_set_sensitive (prev_menuitem, FALSE);
+    gtk_widget_set_sensitive (next_menuitem, FALSE);
   } else {
     gtk_list_store_clear (track_store);
 
@@ -371,6 +373,8 @@ static void update_ui_for_album (AlbumDetails *album)
     gtk_widget_set_sensitive (extract_menuitem, TRUE);
     gtk_widget_set_sensitive (select_all_menuitem, FALSE);
     gtk_widget_set_sensitive (deselect_all_menuitem, TRUE);
+    gtk_widget_set_sensitive (prev_menuitem, FALSE);
+    gtk_widget_set_sensitive (next_menuitem, FALSE);
     
     for (l = album->tracks; l; l=g_list_next (l)) {
       GtkTreeIter iter;
@@ -1044,8 +1048,8 @@ static GtkTreeModel* create_genre_list(void) {
 GtkWidget *
 sj_make_volume_button (void)
 {
-  GtkWidget *w = sj_volume_button_new (0.0, 1.0, 0.02);
-  sj_volume_button_set_value (SJ_VOLUME_BUTTON (w), 1.0);
+  GtkWidget *w = bacon_volume_button_new (0.0, 1.0, 0.02);
+  bacon_volume_button_set_value (BACON_VOLUME_BUTTON (w), 1.0);
   return w;
 }
 
@@ -1081,6 +1085,7 @@ on_message_received (const char *message, gpointer user_data)
 int main (int argc, char **argv)
 {
   GError *error = NULL;
+  GtkTreeSelection *selection;
   struct poptOption options[] = {
     { NULL, '\0', POPT_ARG_INCLUDE_TABLE, NULL, 0, "GStreamer", NULL },
     { "auto-start", 'a', POPT_ARG_NONE, &autostart, 0, "Start extracting immediately", NULL},
@@ -1173,6 +1178,8 @@ int main (int argc, char **argv)
   extract_menuitem = glade_xml_get_widget (glade, "extract_menuitem");
   play_button = glade_xml_get_widget (glade, "play_button");
   play_menuitem = glade_xml_get_widget (glade, "play_menuitem");
+  next_menuitem = glade_xml_get_widget (glade, "next_track_menuitem");
+  prev_menuitem = glade_xml_get_widget (glade, "previous_track_menuitem");
   status_bar = glade_xml_get_widget (glade, "status_bar");
 
   {
@@ -1192,7 +1199,7 @@ int main (int argc, char **argv)
     
     toggle_renderer = gtk_cell_renderer_toggle_new ();
     g_signal_connect (toggle_renderer, "toggled", G_CALLBACK (on_extract_toggled), NULL);
-    column = gtk_tree_view_column_new_with_attributes (_("Extract"),
+    column = gtk_tree_view_column_new_with_attributes ("",
                                                        toggle_renderer,
                                                        "active", COLUMN_EXTRACT,
                                                        NULL);
@@ -1253,6 +1260,11 @@ int main (int argc, char **argv)
     exit (1);
   }
 
+  sj_play_init ();
+
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (track_listview));
+  gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
+	  
   http_proxy_setup (gconf_client);
   /* TODO: port port basepath to baseuri */
   baseuri_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_BASEURI, NULL, TRUE, NULL), NULL);
