@@ -35,11 +35,11 @@
 #include <gtk/gtkstatusbar.h>
 #include <gtk/gtkmessagedialog.h>
 
-#include "sj-volume.h"
+#include "bacon-volume.h"
 #include "sound-juicer.h"
 
 static GstElement *pipeline = NULL;
-static guint id = 0;
+static guint id = 0, button_change_id = 0;
 static gint seek_to_track = -1, current_track = -1;
 static gboolean seeking = FALSE, internal_update = FALSE;
 static guint64 slen = GST_CLOCK_TIME_NONE;
@@ -47,17 +47,39 @@ static gfloat vol = 1.0;
 
 static GtkTreeIter current_iter;
 
+static GtkWidget *play_button, *next_menuitem, *prev_menuitem, *reread_menuitem, *seek_scale, *volume_button, *statusbar;
+
 /**
  * Select track number.
  */
 
-static void
+static gboolean
 select_track (void)
 {
   GstElement *cd;
 
-  if (seek_to_track == -1)
-    return;
+  if (seek_to_track == -1) {
+    gint tracks =
+        gtk_tree_model_iter_n_children (GTK_TREE_MODEL (track_store), NULL);
+
+    seek_to_track = 0;
+    while (seek_to_track < tracks) {
+      gboolean do_play;
+      GtkTreeIter next_iter;
+
+      gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (track_store),
+          &next_iter, NULL, seek_to_track);
+      gtk_tree_model_get (GTK_TREE_MODEL (track_store), &next_iter,
+          COLUMN_EXTRACT, &do_play, -1);
+      if (do_play)
+        break;
+      seek_to_track++;
+    }
+    if (seek_to_track == tracks) {
+      seek_to_track = -1;
+      return FALSE;
+    }
+  }
 
   gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (track_store),
       &current_iter, NULL, seek_to_track);
@@ -70,6 +92,8 @@ select_track (void)
           seek_to_track, seek_to_track + 1));
   current_track = seek_to_track;
   seek_to_track = -1;
+
+  return TRUE;
 }
 
 /**
@@ -80,6 +104,10 @@ static void
 play (void)
 {
   gst_element_set_state (pipeline, GST_STATE_PLAYING);
+  
+  gtk_widget_set_sensitive (next_menuitem, TRUE);
+  gtk_widget_set_sensitive (prev_menuitem, TRUE);
+  gtk_widget_set_sensitive (reread_menuitem, FALSE);
 }
 
 /**
@@ -101,6 +129,10 @@ stop (void)
 {
   if (pipeline != NULL)
     gst_element_set_state (pipeline, GST_STATE_NULL);
+
+  gtk_widget_set_sensitive (next_menuitem, FALSE);
+  gtk_widget_set_sensitive (prev_menuitem, FALSE);
+  gtk_widget_set_sensitive (reread_menuitem, TRUE);
 }
 
 /**
@@ -121,31 +153,42 @@ static gboolean
 cb_hop_track (gpointer data)
 {
   GtkTreeModel *model;
-  gint tracks;
+  gint tracks, next_track = current_track + 1;
+  GtkTreeIter next_iter;
 
   model = GTK_TREE_MODEL (track_store);
   tracks = gtk_tree_model_iter_n_children (model, NULL);
 
-  if (current_track + 2 == tracks) {
+  while (next_track < tracks) {
+    gboolean do_play;
+
+    gtk_tree_model_iter_nth_child (model,
+        &next_iter, NULL, next_track);
+    gtk_tree_model_get (GTK_TREE_MODEL (track_store), &next_iter,
+        COLUMN_EXTRACT, &do_play, -1);
+    if (do_play)
+      break;
+    next_track++;
+  }
+
+  if (next_track >= tracks) {
     stop ();
     seek_to_track = 0;
   } else {
     char *title;
-    seek_to_track = current_track + 1;
+    seek_to_track = next_track;
     gtk_list_store_set (track_store, &current_iter,
         COLUMN_STATE, STATE_IDLE, -1);
     select_track ();
     gtk_list_store_set (track_store, &current_iter,
         COLUMN_STATE, STATE_PLAYING, -1);
-    gtk_tree_model_get (GTK_TREE_MODEL (track_store),
+    gtk_tree_model_get (model,
         &current_iter, COLUMN_TITLE, &title, -1);
     sj_main_set_title (title);
     g_free (title);
     play ();
   }
 
-  /* FIXME: notify treelist so we can add a 'playing' icon */
-
   /* once */
   return FALSE;
 }
@@ -189,7 +232,6 @@ get_label_for_time (gint sec)
 static void
 set_statusbar_pos (gint pos, gint len)
 {
-  static GtkStatusbar *status = NULL;
   static gint prev_pos = 0, prev_len = 0;
   gchar *x, *y, *r;
 
@@ -204,17 +246,13 @@ set_statusbar_pos (gint pos, gint len)
   g_free (x);
   g_free (y);
 
-  if (status == NULL) {
-    status = GTK_STATUSBAR (glade_xml_get_widget (glade, "status_bar"));
-  }
-  gtk_statusbar_push (status, 0, r);
+  gtk_statusbar_push (GTK_STATUSBAR (statusbar), 0, r);
   g_free (r);
 }
 
 static gboolean
 cb_set_time (gpointer data)
 {
-  GtkWidget *scale;
   GstElement *cd;
   GstPad *pad;
   GstFormat fmt = GST_FORMAT_TIME;
@@ -223,14 +261,13 @@ cb_set_time (gpointer data)
   if (seeking)
     return TRUE;
 
-  scale = glade_xml_get_widget (glade, "seek_scale");
   cd = gst_bin_get_by_name_recurse_up (GST_BIN (pipeline), "cd-source");
   pad = gst_element_get_pad (cd, "src");
 
   if (gst_pad_query (pad, GST_QUERY_TOTAL, &fmt, &len) &&
       gst_pad_query (pad, GST_QUERY_POSITION, &fmt, &pos)) {
     internal_update = TRUE;
-    gtk_range_set_value (GTK_RANGE (scale), (gdouble) pos / len);
+    gtk_range_set_value (GTK_RANGE (seek_scale), (gdouble) pos / len);
     set_statusbar_pos (pos / GST_SECOND, len / GST_SECOND);
     slen = len;
     internal_update = FALSE;
@@ -240,46 +277,69 @@ cb_set_time (gpointer data)
   return TRUE;
 }
 
-static void
-cb_state (GstElement * p, GstElementState old_state, GstElementState new_state,
-    gpointer data)
+static gboolean
+cb_change_button (gpointer data)
 {
-  static GtkWidget *seek, *volume, *status, *play;
-  if (seek == NULL || volume == NULL) {
-    seek = glade_xml_get_widget (glade, "seek_scale");
-    volume = glade_xml_get_widget (glade, "volume_button");
-    status = glade_xml_get_widget (glade, "status_bar");
-    play = glade_xml_get_widget (glade, "play_button");
-  }
+  button_change_id = 0;
 
-  if (old_state == GST_STATE_READY && new_state == GST_STATE_PAUSED) {
+  gtk_button_set_label (GTK_BUTTON (data), GTK_STOCK_MEDIA_PLAY);
+
+  /* once */
+  return FALSE;
+}
+
+static gboolean
+idle_state (gpointer data)
+{
+  gint transition = GPOINTER_TO_INT (data);
+
+  if (transition == GST_STATE_READY_TO_PAUSED) {
     char *title;
-    gtk_widget_show (seek);
-    gtk_widget_show (volume);
+    gtk_widget_show (seek_scale);
+    gtk_widget_show (volume_button);
     gtk_list_store_set (track_store, &current_iter,
         COLUMN_STATE, STATE_PLAYING, -1);
     gtk_tree_model_get (GTK_TREE_MODEL (track_store),
         &current_iter, COLUMN_TITLE, &title, -1);
     sj_main_set_title (title);
     g_free (title);
-  } else if (new_state == GST_STATE_READY && old_state == GST_STATE_PAUSED) {
-    gtk_widget_hide (seek);
-    gtk_widget_hide (volume);
-    gtk_statusbar_pop (GTK_STATUSBAR (status), 0);
+  } else if (transition == GST_STATE_PAUSED_TO_READY) {
+    gtk_widget_hide (seek_scale);
+    gtk_widget_hide (volume_button);
+    gtk_statusbar_pop (GTK_STATUSBAR (statusbar), 0);
     slen = GST_CLOCK_TIME_NONE;
     gtk_list_store_set (track_store, &current_iter,
         COLUMN_STATE, STATE_IDLE, -1);
     sj_main_set_title (NULL);
-  } else if (new_state == GST_STATE_PLAYING) {
-    gtk_button_set_label (GTK_BUTTON (play), GTK_STOCK_MEDIA_PAUSE);
+    gtk_statusbar_push (GTK_STATUSBAR (statusbar), 0, "");
+    current_track = -1;
+  } else if (transition == GST_STATE_PAUSED_TO_PLAYING) {
+    gtk_button_set_label (GTK_BUTTON (play_button), GTK_STOCK_MEDIA_PAUSE);
     id = g_timeout_add (100, (GSourceFunc) cb_set_time, NULL);
-  } else if (old_state == GST_STATE_PLAYING) {
-    gtk_button_set_label (GTK_BUTTON (play), GTK_STOCK_MEDIA_PLAY);
+    if (button_change_id) {
+      g_source_remove (button_change_id);
+      button_change_id = 0;
+    }
+  } else if (transition == GST_STATE_PLAYING_TO_PAUSED) {
     if (id) {
       g_source_remove (id);
       id = 0;
     }
+    /* otherwise button flickers on track-switch */
+    button_change_id =
+        g_timeout_add (500, (GSourceFunc) cb_change_button, play_button);
   }
+
+  /* once */
+  return FALSE;
+}
+
+static void
+cb_state (GstElement * p, GstElementState old_state, GstElementState new_state,
+    gpointer data)
+{
+  g_idle_add ((GSourceFunc) idle_state,
+      GINT_TO_POINTER (old_state << 8 | new_state));
 }
 
 /**
@@ -331,7 +391,6 @@ setup (GError **err)
       return FALSE;
     }
 
-    seek_to_track = 0;
     if (!gtk_tree_model_get_iter_first (GTK_TREE_MODEL (track_store), &current_iter))
       g_warning ("Cannot get first iter");
   }
@@ -360,9 +419,28 @@ on_play_activate (GtkWidget *button, gpointer user_data)
 
   if (is_playing ()) {
     pause ();
-  } else if (setup (&err)) {
-    select_track ();
+  } else if (pipeline && GST_STATE (pipeline) == GST_STATE_PAUSED && 
+                (current_track == seek_to_track || seek_to_track == -1)) {
     play ();
+  } else if (setup (&err)) {
+    char *title;
+    if (current_track != -1)
+      gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (track_store),
+          &current_iter, NULL, current_track);
+    gtk_list_store_set (track_store, &current_iter,
+        COLUMN_STATE, STATE_IDLE, -1);
+    if (select_track ()) {
+      gtk_list_store_set (track_store, &current_iter,
+          COLUMN_STATE, STATE_PLAYING, -1);
+      gtk_tree_model_get (GTK_TREE_MODEL (track_store),
+          &current_iter, COLUMN_TITLE, &title, -1);
+      sj_main_set_title (title);
+      g_free (title);
+
+      play ();
+    }
+    else
+      stop ();
   } else {
     GtkWidget *dialog;
 
@@ -394,6 +472,11 @@ on_tracklist_row_activate (GtkTreeView * treeview, GtkTreePath * path,
   if (setup (&err)) {
     char *title;
     seek_to_track = track - 1;
+
+    if (current_track != -1)
+      gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (track_store),
+          &current_iter, NULL, current_track);
+
     gtk_list_store_set (track_store, &current_iter,
         COLUMN_STATE, STATE_IDLE, -1);
     select_track ();
@@ -418,6 +501,70 @@ on_tracklist_row_activate (GtkTreeView * treeview, GtkTreePath * path,
   }
 }
 
+void
+on_next_track_activate(GtkWidget *button, gpointer data)
+{
+  cb_hop_track (NULL);
+}
+
+void
+on_previous_track_activate(GtkWidget *button, gpointer data)
+{
+  GtkTreeModel *model;
+  gint tracks, prev_track = current_track - 1;
+  GtkTreeIter prev_iter;
+
+  model = GTK_TREE_MODEL (track_store);
+  tracks = gtk_tree_model_iter_n_children (model, NULL);
+
+  while (prev_track >= 0) {
+    gboolean do_play;
+
+    gtk_tree_model_iter_nth_child (model,
+        &prev_iter, NULL, prev_track);
+    gtk_tree_model_get (GTK_TREE_MODEL (track_store), &prev_iter,
+        COLUMN_EXTRACT, &do_play, -1);
+    if (do_play)
+      break;
+    prev_track--;
+  }
+
+  if (prev_track < 0) {
+    stop ();
+    seek_to_track = 0;
+  } else {
+    char *title;
+    seek_to_track = prev_track;
+    gtk_list_store_set (track_store, &current_iter,
+        COLUMN_STATE, STATE_IDLE, -1);
+    select_track ();
+    gtk_list_store_set (track_store, &current_iter,
+        COLUMN_STATE, STATE_PLAYING, -1);
+    gtk_tree_model_get (model,
+        &current_iter, COLUMN_TITLE, &title, -1);
+    sj_main_set_title (title);
+    g_free (title);
+    play ();
+  }
+}
+
+void
+on_tracklist_row_selected (GtkTreeView *treeview,
+               gpointer user_data)
+{
+  gint track;
+  GtkTreeSelection *selection = gtk_tree_view_get_selection (treeview);
+
+  if (is_playing ())
+    return;
+
+  if (gtk_tree_selection_get_selected (selection, NULL, &current_iter)) {
+    gtk_tree_model_get (GTK_TREE_MODEL (track_store),
+        &current_iter, COLUMN_NUMBER, &track, -1);
+    seek_to_track = track - 1;
+  }
+}
+
 /**
  * Volume.
  */
@@ -425,7 +572,7 @@ on_tracklist_row_activate (GtkTreeView * treeview, GtkTreePath * path,
 void
 on_volume_changed (GtkWidget * volb, gpointer data)
 {
-  vol = sj_volume_button_get_value (SJ_VOLUME_BUTTON (volb));
+  vol = bacon_volume_button_get_value (BACON_VOLUME_BUTTON (volb));
   if (pipeline) {
     GstElement *volume;
 
@@ -449,7 +596,6 @@ on_seek_press (GtkWidget * scale, GdkEventButton * event, gpointer user_data)
 void
 on_seek_moved (GtkWidget * scale, gpointer user_data)
 {
-  GtkWidget *s = glade_xml_get_widget (glade, "status_bar");
   gdouble val = gtk_range_get_value (GTK_RANGE (scale));
   gchar *x, *m;
 
@@ -459,7 +605,7 @@ on_seek_moved (GtkWidget * scale, gpointer user_data)
   x = get_label_for_time (slen * val / GST_SECOND);
   m = g_strdup_printf (_("Seeking to %s"), x);
   g_free (x);
-  gtk_statusbar_push (GTK_STATUSBAR (s), 0, m);
+  gtk_statusbar_push (GTK_STATUSBAR (statusbar), 0, m);
   g_free (m);
 }
 
@@ -478,3 +624,15 @@ on_seek_release (GtkWidget * scale, GdkEventButton * event, gpointer user_data)
 
   return FALSE;
 }
+
+void
+sj_play_init (void)
+{
+  play_button = glade_xml_get_widget (glade, "play_button");
+  next_menuitem = glade_xml_get_widget (glade, "next_track_menuitem");
+  prev_menuitem = glade_xml_get_widget (glade, "previous_track_menuitem");
+  reread_menuitem = glade_xml_get_widget (glade, "re-read");
+  seek_scale = glade_xml_get_widget (glade, "seek_scale");
+  volume_button = glade_xml_get_widget (glade, "volume_button");
+  statusbar = glade_xml_get_widget (glade, "status_bar");
+}
